//1秒のクロック生成
module m_1s_clk(input clk,output clk1s);
	reg [24:0] cnt;
	reg r1s;
	wire wcout;
	
	assign wcout=(cnt==25'd24999999) ? 1'b1 : 1'b0;
	assign clk1s=r1s;
	always @(posedge clk) begin
		if(wcout==1'b1) begin
			cnt=0;
			r1s=~r1s;
		end
		else begin
			cnt=cnt+1;
		end
	end
		
endmodule	

//キッチンタイマモジュール
module m_kitchen_timer(
	input clk,				//1秒のクロック
	input mode,				//0:時間のセット,1:タイマ起動
	input mset_btn,		//分の設定
	input sset_btn,		//秒の設定
	output [7:0] min,		//BCD2桁の分
	output [7:0] sec,		//BCD2桁の秒
	output time_end		//時間終了
	);
	wire [3:0] w_cb;		//キャリー／ボロー
	wire w_cb_sm;			//秒から分へのキャリーボロー
	wire m_clk;				//分のクロック
	wire s_clk;				//秒のクロック
	wire w_cb_in;			//最下位のキャリーイン
	wire r_run;				//カウントダウンスタート
	wire iclk;				//内部クロック
	wire clken;				//カウントクロックの有効ビット
	
	assign iclk=r_run & clk;
	assign clken=~w_cb[3];
	m_rs_flipflop r0(~sset_btn,mode,r_run);

	
	assign w_cb_sm=(mode==1'b1) ? w_cb[1] : 1'b1;
	assign m_clk=(mode==1'b1) ? iclk : mset_btn;
	assign s_clk=(mode==1'b1) ? iclk : sset_btn;
	assign w_cb_in=(mode==1'b1) ?r_run : 1'b1;
	assign time_end=(mode & w_cb[3]);
	m_up_down_counter #(9) u0(s_clk,clken,mode,w_cb_in,w_cb[0],sec[3:0]);	//1秒
	m_up_down_counter #(5) u1(s_clk,clken,mode,w_cb[0],w_cb[1],sec[7:4]);	//10秒
	m_up_down_counter #(9) u2(m_clk,clken,mode,w_cb_sm,w_cb[2],min[3:0]);	//1秒
	m_up_down_counter #(5) u3(m_clk,clken,mode,w_cb[2],w_cb[3],min[7:4]);	//10秒
	
	
endmodule


